/*!**************************************************************************
\file    read_brdf.cpp
\author  Jiri Filip, UTIA AS CR
\date    September 2014
\version 1.00

The main file for the UTIA BRDF data loading and value interpolation

################################################################
compile using:
   g++ TBRDF.cpp lodepng.cpp brdf_read.cpp -o brdf_read -O3
run using:
   ./brdf_read
   (BRDF bin files are assumed to be in subdir 'data/')
################################################################

lodePNG library:
Copyright (c) 2005-2014 Lode Vandevenne
******************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <vector>
#include <iostream>

#include "TBRDF.h"    // loading BRDF and interpolation
#include "lodepng.h"  // for PNG image saving

#define PI 3.1415926535897932384626433832795

void savePNG(const char* filename, std::vector<unsigned char>& image, unsigned width, unsigned height)
{
  unsigned error = lodepng::encode(filename, image, width, height);
  if(error) std::cout << "encoder error " << error << ": "<< lodepng_error_text(error) << std::endl;
}//--- savePNG -----------------------------------------------------


int main(int argc, char *argv[])
{  
  float d2r = PI/180;
  double RGB[3];

  //################################################################
  // EXAMPLE 1 - basic loading of one material and value lookup
  //################################################################
  BRDF B;
  B.load_brdf("data/m003_carpet01.bin");
  float theta_i =  50; // illumination elevation
  float phi_i   = 100; // illumination azimuth
  float theta_v =  50; // view elevation
  float phi_v   =   0; // view azimuth

  B.lookup_brdf_val(theta_i*d2r, phi_i*d2r, theta_v*d2r, phi_v*d2r, RGB);
  printf("RGB: %f %f %f\n",RGB[0],RGB[1],RGB[2]);

  //################################################################
  // EXAMPLE 2 - restoring azimuthal subspace for fixed elevation angles
  //             theta_i=60/theta_v=60 and saving to PNG
  //################################################################
  theta_i =  50; // illumination elevation
  theta_v =  50; // view elevation
  const int NoOfMaterials = 150;

  // loading material names ================================
  char *materialNames[NoOfMaterials]; 
  for(int m=0;m<NoOfMaterials;m++)
    materialNames[m] = new char [100];
  
  FILE *fpr;
  if((fpr=fopen("material_names.txt","r"))==NULL)
  {
     printf(" Error when opening file !! Exiting...\n");
     exit(0);
  }
  for(int i=0;i<NoOfMaterials;i++)
    int status = fscanf(fpr,"%s\n",materialNames[i]);
  fclose(fpr);

  // restoring subspace images for all materials ===========
  // images show azimuthally dependent anisotropic behavior

  int nr = 360;   // size of subspace images 360 = azimuthal resolution 1 degree
  double c = 300; // multiplying constant

  std::vector<unsigned char> image;
  image.resize(nr*nr*4);
  char fileName[100];

  for(int m=0;m<NoOfMaterials;m++)
    {
      sprintf(fileName,"data/%s.bin",materialNames[m]);
      B.load_brdf(fileName);

      for(int phi_i=0;phi_i<nr;phi_i++)
        for(int phi_v=0;phi_v<nr;phi_v++)
          {
            B.lookup_brdf_val(theta_i*d2r, (float)phi_i*d2r, theta_v*d2r, (float)phi_v*d2r, RGB);
            for(int isp=0;isp<3;isp++)
              {
                double aux = c*RGB[isp];
                // clamping range to 0-255 for 8-bit PNG
                if(aux<0)   aux = 0.f;
                if(aux>255) aux = 255.f;
                image[4*nr*phi_i + 4*phi_v + isp] = (unsigned char)aux;
              }
            image[4*nr*phi_i + 4*phi_v + 3] = 255;
          }
      sprintf(fileName,"data/pipv_%s.png",materialNames[m]);
      savePNG(fileName, image, nr, nr);
    }

  for(int m=0;m<NoOfMaterials;m++)
    delete [] materialNames[m];

  return 0;

}//--- main ---------------------------------------------------------------------


